use serde::Serialize;

use super::render_context::RenderContext;
use crate::error::CustomError;
use crate::intermediate::IDocumentElement;

use super::macros::render;
use super::RenderHeading;
use super::RenderSection;

#[derive(Debug, Serialize)]
#[serde(untagged)]
pub(crate) enum RenderDocumentElement {
    Heading(RenderHeading),
    Section(RenderSection),
}

render!(
    RenderDocumentElement,
    IDocumentElement,
    original,
    render_context,
    {
        match original {
            IDocumentElement::Heading(inner) => Ok(RenderDocumentElement::Heading(
                RenderHeading::new(render_context.clone(), inner)?,
            )),
            IDocumentElement::Section(inner) => Ok(RenderDocumentElement::Section(
                RenderSection::new(render_context.clone(), inner)?,
            )),
        }
    }
);
