use serde::Serialize;

use super::render_context::RenderContext;
use crate::error::CustomError;
use crate::intermediate::IHeading;

use super::macros::render;
use super::RenderDocumentElement;
use super::RenderObject;

#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "heading")]
pub(crate) struct RenderHeading {
    level: organic::types::HeadlineLevel,
    title: Vec<RenderObject>,
    children: Vec<RenderDocumentElement>,
    post_blank: organic::types::PostBlank,
}

render!(RenderHeading, IHeading, original, render_context, {
    let title = {
        let mut ret = Vec::new();
        for obj in original.title.iter() {
            ret.push(RenderObject::new(render_context.clone(), obj)?);
        }
        ret
    };

    let children = {
        let mut ret = Vec::new();
        for obj in original.children.iter() {
            ret.push(RenderDocumentElement::new(render_context.clone(), obj)?);
        }
        ret
    };

    Ok(RenderHeading {
        level: original.level + 1, // Adding 1 because the page title is going to be h1.
        title,
        children,
        post_blank: original.post_blank,
    })
});
