use serde::Serialize;

/// The header that goes above the content of the page.
///
/// This header will be mostly the same on every page.
#[derive(Debug, Serialize)]
pub(crate) struct PageHeader {
    website_title: Option<String>,
    home_link: Option<String>,
    nav_links: Vec<NavLink>,
}

/// A link in the top-right of the page.
#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "nav_link")]
pub(crate) struct NavLink {
    text: Option<String>,
    url: Option<String>,
}

impl PageHeader {
    pub(crate) fn new(
        website_title: Option<String>,
        home_link: Option<String>,
        about_me_link: Option<String>,
    ) -> PageHeader {
        PageHeader {
            website_title,
            home_link,
            nav_links: about_me_link
                .map(|url| {
                    vec![NavLink {
                        text: Some("About Me".to_owned()),
                        url: Some(url),
                    }]
                })
                .unwrap_or_default(),
        }
    }
}
