use serde::Serialize;

use super::render_context::RenderContext;
use crate::error::CustomError;
use crate::intermediate::IParagraph;

use super::macros::render;
use super::RenderObject;

#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "paragraph")]
pub(crate) struct RenderParagraph {
    children: Vec<RenderObject>,
    is_single_image: bool,
    post_blank: organic::types::PostBlank,
}

render!(RenderParagraph, IParagraph, original, render_context, {
    let children = {
        let mut ret = Vec::new();
        for obj in original.children.iter() {
            ret.push(RenderObject::new(render_context.clone(), obj)?);
        }
        ret
    };

    Ok(RenderParagraph {
        children,
        is_single_image: original.is_single_image(),
        post_blank: original.post_blank,
    })
});
