use serde::Serialize;

use super::render_context::RenderContext;
use crate::error::CustomError;
use crate::intermediate::IPlainList;

use super::macros::render;
use super::plain_list_item::RenderPlainListItem;

#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "plain_list")]
pub(crate) struct RenderPlainList {
    list_type: String,
    children: Vec<RenderPlainListItem>,
    post_blank: organic::types::PostBlank,
}

render!(RenderPlainList, IPlainList, original, render_context, {
    let list_type = match original.list_type {
        organic::types::PlainListType::Unordered => "unordered".to_owned(),
        organic::types::PlainListType::Ordered => "ordered".to_owned(),
        organic::types::PlainListType::Descriptive => "descriptive".to_owned(),
    };
    let children = {
        let mut ret = Vec::new();
        for obj in original.children.iter() {
            ret.push(RenderPlainListItem::new(render_context.clone(), obj)?);
        }
        ret
    };

    Ok(RenderPlainList {
        list_type,
        children,
        post_blank: original.post_blank,
    })
});
