use serde::Serialize;

use super::render_context::RenderContext;
use crate::error::CustomError;
use crate::intermediate::IUnderline;

use super::macros::render;
use super::RenderObject;

#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "underline")]
pub(crate) struct RenderUnderline {
    children: Vec<RenderObject>,
    post_blank: organic::types::PostBlank,
}

render!(RenderUnderline, IUnderline, original, render_context, {
    let children = {
        let mut ret = Vec::new();
        for obj in original.children.iter() {
            ret.push(RenderObject::new(render_context.clone(), obj)?);
        }
        ret
    };

    Ok(RenderUnderline {
        children,
        post_blank: original.post_blank,
    })
});
