use super::macros::intermediate;
use super::IObject;
use crate::error::CustomError;
use organic::types::StandardProperties;

/// Special case for list items with only paragraphs and sublists as their children. In those cases, the paragraph tags are omitted. This is equivalent to a Paragraph but in a different struct to have a different type tag.
#[derive(Debug, Clone)]
pub(crate) struct IPlainListSimpleItem {
    pub(crate) children: Vec<IObject>,
    pub(crate) post_blank: organic::types::PostBlank,
}

intermediate!(
    IPlainListSimpleItem,
    &'orig organic::types::Paragraph<'parse>,
    original,
    intermediate_context,
    {
        let children = {
            let mut ret = Vec::new();
            for obj in original.children.iter() {
                ret.push(IObject::new(intermediate_context.clone(), obj).await?);
            }
            ret
        };

        Ok(IPlainListSimpleItem {
            children,
            post_blank: original.get_post_blank(),
        })
    }
);
